import com.fwa.debug.Tracer;
import com.fwa.events.ListenerBroadcaster;
import com.fwa.events.IListenable;
import com.fwa.ui.Placeholders;
import com.fwa.ui.DepthManager;
import com.fwa.ui.Window;
import com.fwa.ui.IPreloaderListener;
import mx.utils.Delegate;

/**
* @class 	Windows
* This class manages the creation of Windows and Placeholders.
* Each Placeholder created is loaded with a DepthManager instance to delegate control
* of its inner Windows.
* Each Window created is placed into a Placeholder and added to a DepthManager
* This class also uses IPreloaderListener as a common interface to attach a preloader to a Window
* This class also broadcasts Windows events, like onWindowClosed, Shown, Hiden, etc.
* @author 	Federico Campo Piombi
* @version 	1.5
* 
* @see 		Window
* @see 		IPreloaderListener
* @see 		DepthManager
* @see 		Collection
*/
class com.fwa.ui.Windows implements IListenable {
// properties:
	
	private var Broadcaster:ListenerBroadcaster; //stores an instance of the ListenerBroadcaster Class

	/**
	* @property _instance
	* @tooltip 	Stores the Singleton instance
	* @type 	WindowsManager
	*/
	private static var _instance:Windows = null;
	


	
//CONSTRUCTOR: 
	private function Windows() {
		Broadcaster = new ListenerBroadcaster()
	}

	
// PUBLIC methods:

	/**
	* @method 	getInstance
	* @tooltip 	Resturns a reference to the Singleton WindowsManager class
	* @return 	WindowsManager
	*/
	public static function getInstance():Windows {
		if(Windows._instance == null){
			Windows._instance = new Windows()
		}
		return(Windows._instance)
	}
	
	
	/**
	* @method 	createWindow
	* @tooltip 	Creates a Window in the given Placeholder and returns its reference
	* @param 	ph (Object) Placeholder where the window will be created
	* @param 	identifier (String) window identifier
	* @param 	modal (Boolean) Wether this Window should be modal or not
	* @return 	Window
	*/
	public function createWindow(ph:Object , identifier:String, modal:Boolean):Window {
		var wnd:Window = Placeholders.getWindow(ph.placeholder, retrieveWindowIdentifier(identifier))
		if(wnd==undefined){
			var depth:Number = ph.depthMGR.getHighestDepth()
			debug("createWindow", "Creating Window:"+identifier+" at:"+ph.placeholder+" on depth:"+depth)
			wnd = new Window(ph.placeholder, identifier, depth, modal)
		}
		return(wnd)
	}
	
	
	/**
	* @method 	getWindow
	* @tooltip 	Given a placeholder identifier, and a window identifier, returns a reference to that Window
	* @param 	placeholder_id (String) placeholder identifier
	* @param 	window_filename (String) filename loaded into that window (or identifier)
	* @return 	Window
	*/
	public function getWindow(placeholder_id:String, window_filename:String):Window {
		var window_id = retrieveWindowIdentifier(window_filename)
		return ( Placeholders.getWindow(placeholder_id, window_id) )
	}
	
	
	/**
	* @method 	open
	* @tooltip 	Opens a Window in a particular PlaceHolder, with given configuration. 
	* 		   	Returns a reference to the new Window.
	* @param 	placeholder (String) target PlaceHolder where the Window is
	* @param 	config (Object) Object with configuration params
	* @return 	Window
	*/
	public function open(placeholder:String, config:Object):Window {
		if(validateWindowConfiguration(config) == true){
			//find placeholder in the collection
			var ph:Object = Placeholders.getPlaceholder(placeholder)
			var wnd:Window
			
			//if found placeholder
			if(ph!==undefined){
				//first check if window exists in placeholder
				if(Placeholders.windowExists(placeholder, config.identifier)){
					//if window exists and overwrite is false, bring it to front
					var existant_window:Window = getWindow(placeholder, config.identifier)
					debug("open", "Window exists:"+config.identifier+" as:"+existant_window._name)
					ph.depthMGR.occupyLast(existant_window.getContainerClip())
					
					return(existant_window)
				}else{
					//create new window
					wnd = createWindow(ph, config.identifier, config.modal)
					
					//register the Window to this Placeholder
					registerWindow(wnd, ph)
					
					//load content into window
					loadWindowContent(wnd, ph, config)
					
					return(wnd)
				}
				
			}else{
				debug("open", "Invalid Placeholder:"+placeholder)
				return(undefined)
			}
		}else{
			debug("open", "Invalid Window configuration")
			return(undefined)
		}
	}
	
	
	/**
	* @method 	addWindow
	* @tooltip 	creates a new Window based on an existant MovieClip, and manages it
	* @param 	clip (MovieClip) MovieClip to associate to a new Window
	* @param 	placeh_id (String) (optional) Placeholder where to place this new Window (default __WINDOWS)
	* @return 	Window
	*/
	public function addWindow(clip:MovieClip, placeh_id:String):Window {
		debug("addWindow", "adding:"+clip)
		var ph:Object = (placeh_id!==undefined)? Placeholders.getPlaceholder(placeh_id) : Placeholders.getPlaceholder("__WINDOWS")
		var depth:Number = ph.depthMGR.getHighestDepth()
		
		var wnd:Window = new Window(ph.placeholder, clip._name, depth, false)
		wnd.associateContent(clip)
		var wnd_lst:Object = new Object()
		wnd_lst.onClose = Delegate.create(this,onWindowClosed)
		wnd_lst.onShow = Delegate.create(this, onWindowShown)
		wnd_lst.onHide = Delegate.create(this, onWindowHiden)
		
		ph.depthMGR.occupyLast(wnd.getContainerClip())
		return(wnd)
	}
	
	
	/**
	* @method 	close
	* @tooltip 	Closes a Window and unregisters it from the DepthManager. Returns TRUE if success.
	* @param 	placeholder_id (Window) Name of the Placeholder where the Window is
	* @param 	wnd (Window) window to close and unregister from the DepthManager
	* @return 	Boolean
	*/
	public function close(placeholder_id:String, wnd:Window):Boolean {
		//check if Placeholder and Window actually exists, and then
		//remove Window from Placeholder's Collection and depthManager
		
		debug("close", "Closing ph:"+placeholder_id+" wnd:"+wnd._name)
		
		var ph:Object = Placeholders.getPlaceholder(placeholder_id)
		if(ph !== undefined){
			if(wnd.getContainerClip() !==undefined){
				unregisterWindow(wnd, Placeholders.getPlaceholder(placeholder_id))
				wnd.close()
				return(true)
			}else{
				debug("close", "Window doesn't appear to exist!")
				return(false)
			}
		}else{
			debug("close", "Placeholder doesn't appear to exist!")
		}
	}
	
	
	/**
	* @method 		closeAllFromPlaceholder
	* @tooltip 		Closes all windows from a given PlaceHolder
	* @description	Start closing from the last added element to the first.
	* 				This avoids removing the first element, and skipping the new element in that position
	* @param 		placeholder_id (String) placeholder where windows will be closed
	* @return 		Void
	*/
	public function closeAllFromPlaceholder(placeholder_id:String):Void {
		var ph:Object = Placeholders.getPlaceholder(placeholder_id)
		if(ph!==undefined){
			var places:Array = ph.places.getElements()
			var places_count:Number = places.length
			for(var j:Number = 0; j<places_count; j++){
				close(placeholder_id, places[places_count-(j+1)].window)
			}
			debug("closeAllFromPlaceholder", "PLACES LEFT:"+places.length)
		}else{
			debug("closeAllFromPlaceholder", "Invalid Placeholder: "+placeholder_id)
		}
	}
	
	
	/**
	* @method 	closeAll
	* @tooltip 	Closes all windows from all registered placeholders
	* @return 	Void
	*/
	public function closeAll():Void {
		var plhs:Array = Placeholders.getElements()
		for(var i:Number = 0; i<plhs.length; i++){
			closeAllFromPlaceholder(plhs[i].identifier)
		}
	}
	
	
	/**
	* @method 	hidePlaceholder
	* @tooltip 	Hides all windows from a given PlaceHolder
	* @param 	placeholder_id (String) placeholder where windows will be hide
	* @return 	Void
	*/
	public function hidePlaceholder(placeholder_id:String):Void {
		var ph:Object = Placeholders.getPlaceholder(placeholder_id)
		if(ph!==undefined){
			ph.placeholder._visible = false
		}else{
			debug("hidePlaceholder", "Invalid Placeholder: "+placeholder_id)
		}
	}
	
	
	/**
	* @method 	showPlaceholder
	* @tooltip 	Shows all windows from a given PlaceHolder
	* @param 	placeholder_id (String) placeholder where windows will be shown
	* @return 	Void
	*/
	public function showPlaceholder(placeholder_id:String):Void {
		var ph:Object = Placeholders.getPlaceholder(placeholder_id)
		if(ph!==undefined){
			ph.placeholder._visible = true
		}else{
			debug("hidePlaceholder", "Invalid Placeholder: "+placeholder_id)
		}
	}
	
	
	/**
	* @method 	retrieveWindowIdentifier
	* @tooltip 	creates a Window Identifier from a filename Sintax, stripping slashes (/ and \) and dots
	* @param 	filename (String) filename to convert
	* @return 	String
	*/
	public function retrieveWindowIdentifier(filename:String):String {
		var identifier:String 
		identifier = (filename.indexOf("../")!== -1)? filename.split("../").join("______") : filename
		identifier = (identifier.indexOf("./")!==-1)? identifier.split("./").join("_____") : identifier
		identifier = (identifier.indexOf("/")!== -1)? identifier.split("/").join("____") : identifier
		identifier = (identifier.indexOf(".")!== -1)? identifier.split(".").join("___") : identifier
		identifier = (identifier.indexOf("-")!== -1)? identifier.split("-").join("__") : identifier
		return(identifier)	
	}
	
	
	/**
	* @method 	retrieveWindowFilename
	* @tooltip 	retrieves the Filename Sintax from a Window Identifier Sintax
	* @param 	identifier (String) Window identifier to convert
	* @return 	String
	*/
	public function retrieveWindowFilename(identifier:String):String {
		var filename:String
		filename = (identifier.indexOf("______")!== -1)? identifier.split("______").join("../") : identifier
		filename = (filename.indexOf("_____")!== -1)? filename.split("_____").join("./") : filename
		filename = (filename.indexOf("____")!== -1 )? filename.split("____").join("/") : filename
		filename = (filename.indexOf("___")!== -1)? filename.split("___").join(".") : filename
		filename = (filename.indexOf("__")!== -1)? filename.split("__").join("-") : filename
		return(filename)
	}
	
	

// PRIVATE methods
	
	
	private function loadWindowContent(wnd:Window, ph:Object, config:Object):Void {
		//load/attach window content
		if(config.filename!==undefined){
			wnd.loadContent(config.filename)
		}else{
			wnd.attachContent(config.linkage)
		}
		
		//fire the onWindowOpened event
		Broadcaster.broadcastMessage("onWindowOpened",[wnd, ph.placeholder])
	}
	
	/**
	* @method 	registerWindow
	* @tooltip 	Registers a Window to a specific Placeholder:
	* 			adds the Window to the Placeholder's DepthManager and Collection
	* @param 	wnd (Window) Window to register
	* @param 	ph (Object) placeholder where to register Window
	* @return 	Void
	*/
	private function registerWindow(wnd:Window, ph:Object):Void {
		debug("registerWindow", "wnd:"+wnd._name)
		//add window to the Placeholder's depthManager
		ph.depthMGR.occupyLast(wnd.getContainerClip())
		//ph.depthMGR.showAll() //debug
		
		//add window to the Placeholder's Places
		ph.places.addElement({identifier:wnd._name, window:wnd})
		
		//register the onClose, onShow & onHide Window events
		var wnd_lst:Object = new Object()
		wnd_lst.onClose = Delegate.create(this,onWindowClosed)
		wnd_lst.onShow = Delegate.create(this, onWindowShown)
		wnd_lst.onHide = Delegate.create(this, onWindowHiden)
		wnd.addListener(wnd_lst)
	}
	
	
	/**
	* @method 	unregisterWindow
	* @tooltip 	Unregisters the given Window from the specified Placeholder
	* @param 	wnd (Window) Window to register
	* @param 	ph (Object) placeholder where to register Window
	* @return 	Void
	*/
	private function unregisterWindow(wnd:Window, ph:Object):Void {
		if(Placeholders.windowExists(ph.identifier, wnd._name)==true){
			ph.depthMGR.remove(wnd.getContainerClip())
			//ph.depthMGR.showAll() //debug
			
			var elemToRemove:Object = ph.places.findElement("identifier", wnd._name)
			ph.places.removeElement( elemToRemove )
			debug("unregisterWindow", "WND Unregistered OK")
		}else{
			debug("unregisterWindow", "WND Inexistente")
		}
	}
	
	
	/**
	* @method 	validateWindowConfiguration
	* @tooltip 	Validates a WindowsManager.open() config object
	* @param 	config (Object) configuration object to validate
	* @return 	Boolean
	*/
	private function validateWindowConfiguration(config:Object):Boolean {
		var errors:Number = 0
		
		if(config.identifier==undefined || config.modal==undefined){
			debug("validateWindowConfiguration", "error: wrong Identifier or Modal")
			trace("identifier:"+config.identifier+" modal:"+config.modal)
			errors++
		}
		if(config.filename==undefined && config.linkage==undefined){
			trace("filename:"+config.filename)
			trace("linkage:"+config.linkage)
			debug("validateWindowConfiguration", "error: required Filename or Linkage")
			errors++
		}
		
		return ( (errors==0)? true : false)
	}
	
	
	
	/**
	* @method 	debug
	* @tooltip 	Traces a message to the output panel
	* @param	method (Strign) Method that invokes <debug>
	* @param	msg (String) message to output
	*/
	private function debug(method:String, error:String){
		//Tracer.output(error, method, "Windows")
	}

// events:

	/**
	* @event onWindowOpened [Broadcasted]
	*/

	/**
	* @event 		onWindowClosed
	* @tooltip 		event fired when a Window/Popup/Alert is closed. 
	* @description 	This event is executed before the Window's MovieClip is removed from the Stage, 
	* 				this way it can be previously unregistered from the Placeholder.
	* 				Also all listeners receiving the onWindowClosed event, will be receiving a Window
	* 				reference, because at this point the Window's MovieClip still exists.
	* 				For more info see: com.Wunderman.UI.Window > close()
	* @param 		wnd (Window) Window being closed
	* @return 		Void
	*/
	private function onWindowClosed(wnd:Window):Void {
		debug("onWindowClosed", "wnd:"+wnd._name)
		//DESREGISTRAR DEL PLACHOLDER ANTES DE HACER EL BROADCAST
		unregisterWindow(wnd, Placeholders.getWindowPlaceholder(wnd))
		Broadcaster.broadcastMessage("onWindowClosed",[wnd])
	}

	/**
	* @event 	onWindowShown
	* @param 	wnd (Window) Window being shown
	* @tooltip 	event fired when a Window/Popup/Alert is show
	*/
	private function onWindowShown(wnd:Window):Void {
		Broadcaster.broadcastMessage("onWindowShown",[wnd])
	}

	/**
	* @event 	onWindowHiden
	* @param 	wnd (Window) Window being hiden
	* @tooltip 	event fired when a Window/Popup/Alert is hiden
	*/
	private function onWindowHiden(wnd:Window):Void {
		Broadcaster.broadcastMessage("onWindowHiden", [wnd])
	}

	
// listener broadcaster integration:
	/**
	* @method 	addListener
	* @tooltip 	adds a new listener to the Broadcaster's collection
	* @param 	listener_obj (Object) object to add to the Collection
	* @return 	Boolean
	*/
	public function addListener(listener_obj:Object):Boolean {
		return( Broadcaster.addListener(listener_obj) )
	}

	/**
	* @method 	removeListener
	* @tooltip 	removes a listener from the Broadcaster's collection
	* @param 	listener_obj (Object) listener to be removed
	* @return 	Boolean
	*/
	public function removeListener(listener_obj:Object):Boolean {
		return( Broadcaster.removeListener(listener_obj) )
	}

}
// ** END Windows ***************************************************