import com.fwa.events.IListenable;
import com.fwa.events.ListenerBroadcaster;
import com.fwa.data.ObservableSound;
import mx.utils.Delegate;

/**
* SoundLoader Class
* Creates a process to load sounds sequentially.
* Requires ObservableSound as sound inputs (because of its Broadcaster properties).
* 
* Broadcasts:
* 
* Message: 
* <onProcessStart> When the SoundLoader Queue begins to load the first item
* Params:
* <li>items (Number) The total of items to load
* 
* Message: 
* <onProcessProgress> As the Queue processes each item. Sends the overall progress of the queue, and the load progress of the current item 
* Params:
* <li>overall_percent (Number) The progress percent of the overall queue
* <li>item_percent (Number) The progress percent of the current loading item
* 
* Message: 
* <onProcessComplete> When the last item of the Queue has finished loading
* Params:
* <li>none</li>
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.2
* @date 	21 Nov 2006
*/
class com.fwa.loaders.SoundLoader implements IListenable {
	
	private var broadcaster:ListenerBroadcaster
	private var idle:Boolean
	private var current_item:Object
	private var items:Array
	private var total_items:Number   //used to calculate the overall percent
	
	/**
	* Constructor
	*/
	public function SoundLoader() {
		broadcaster = new ListenerBroadcaster()
		items = new Array()
		idle = true
	}
	
	/**
	* Adds a Sound to the Queue. If the Queue is not idle by the time an item is being added, returns FALSE.
	* @param	filename (String) the name of the file to load
	* @param	soundObject (ObservableSound) the sound object where the file is being loaded
	* @return	Boolean
	*/
	public function addSound(filename:String, soundObject:ObservableSound):Boolean {
		if(idle){
			var sobj:Object = new Object({file:filename, sound:soundObject})
			items.push(sobj)
			total_items = items.length
			return(true)
		}else{
			return(false)
		}
	}
	
	/**
	* Processes the Sounds Queue (if Idle).
	* If the Queue has no elements to process, broadcasts the "onProcessCompleted" message
	* @return	Void
	*/
	public function process():Void {
		if(idle){
			if(items.length>0){
				idle = false
				current_item = items.pop()
				current_item.sound.addListener(getSoundListener())
				current_item.sound.loadSound(current_item.file)
			}else{
				broadcaster.broadcastMessage("onProcessComplete")
				total_items = 0
			}
		}
	}
	
	/**
	* Registers the "onLoadStart" message broadcasted by the current loading item.
	* Broadcasts onProcessStart
	* @param	snd (ObservableSound) the sound being loaded
	* @return	Void
	*/
	private function onItemLoadStart(snd:ObservableSound):Void {
		if(items.length == total_items-1){
			broadcaster.broadcastMessage("onProcessStart", [total_items])
		}
	}
	
	/**
	* Registers the "onLoadProgress" message broadcasted by the current loading item.
	* Calculates the overall progress of the Queue and broadcasts the "onProcessProgress" message
	* @param	snd (ObservableSound) the sound being loaded
	* @param	bL (Number) the total bytes loaded
	* @param	bT (Number) the total bytes
	* @return	Void
	*/
	private function onItemLoadProgress(snd:ObservableSound, bL:Number, bT:Number):Void {
		var current_item:Number = total_items - items.length
		var current_item_percent:Number = Math.ceil(bL * 100 / bT)
		var single_item_percent:Number = Math.ceil(100 / total_items)
		var current_item_overall_percent:Number = single_item_percent * current_item_percent / 100
		var previous_items_overall_percent:Number = (current_item-1) * single_item_percent
		var overall_percent:Number = Math.ceil(previous_items_overall_percent + current_item_overall_percent)
		
		overall_percent = (overall_percent>100)? 100 : overall_percent;
		trace("overall percent:"+overall_percent);
		
		broadcaster.broadcastMessage("onProcessProgress", [overall_percent, current_item_percent])
	}
	
	/**
	* Registers the "onLoadComplete" message broadcasted by the current loading item.
	* Proceeds to the next item in the queue
	* @param	snd (ObservableSound) the sound being loaded
	* @param	success (Boolean) true if the sound loaded properly
	* @param	duration (Number) the sound's duration
	* @return	Void
	*/
	private function onItemLoadComplete(snd:ObservableSound, success:Boolean, duration:Number):Void {
		idle = true
		process()
	}
	
	/**
	* Returns a Listener Object for the ObservableSound being loaded
	* @return	Object
	*/
	private function getSoundListener():Object {
		var slst:Object = new Object();
		slst.onLoadStart = Delegate.create(this, onItemLoadStart)
		slst.onLoadProgress = Delegate.create(this, onItemLoadProgress)
		slst.onLoadComplete = Delegate.create(this, onItemLoadComplete)
		return(slst)
	}
	
	
	//listener-broadcaster integration
	/**
	* @see	com.fwa.events.ListenerBroadcaster
	*/
	public function addListener(obj:Object):Boolean {
		return(broadcaster.addListener(obj))
	}
	
	/**
	* @see	com.fwa.events.ListenerBroadcaster
	*/
	public function removeListener(obj:Object):Boolean {
		return(broadcaster.removeListener(obj))
	}
}