/**
* Static class IntervalManager
* This class provides a centralized and cleaner way to manage function Intervals.
* It also provides a method called <setTimeOut> to create a single-time-execute Interval.
* 
* @author	Michael Avila
* @version	1.0.0
* @usage
*  IntervalManager.setInterval("hello", this, "sayHello", 1000, ["Michael"]);
*  IntervalManager.setTimeout("helloOnce", this, "sayHelloOnce", 1000);
*
*  function sayHello(args:Array):Void {
*    trace("Hello, " + args[0]);
*  }
*
*  function sayHelloOnce() : Void {
*    trace( "Hello from the TimeOut" );
*  }
*
*  function onMouseDown() {
*    IntervalManager.clearInterval( "hello" );
*  }
*/

class com.fwa.utils.IntervalManager {
	/**
	* Stores the interval ID's in a dictionary, so that you can name them
	*/
	private static var intervalIDs : Object = new Object();

	/**
	* Sets an interval, storing it's reference in this IntervalManager.
	* If there is already an interval associated with the name that you provide
	* (meaning an interval that was created with but not cleared with the IntervalManager),
	*  the IntervalManager will clear the interval and set the interval you provide
	* in it's place.
	*
	* @param 	name (String) The name you are assigning the interval, note that this is how you will be deleting it
	* @param 	path (Object) The path to the function that will be called by the setInterval
	* @param 	functionName (String) The name of the function as a string
	* @param 	interval (Number) The interval that this function will be called on
	* @param 	args (Array) The args that will be passed to the interval.  The arguments are passed as an array, so be sure
	* 						 to have your function compensate for that.
	* @return	Void
	*/
	public static function setInterval(name:String, path:Object, functionName:String, interval:Number, args:Array):Void {
		if(IntervalManager.intervalIDs[name] != undefined) {
			IntervalManager.clearInterval( name );
		};
		
		//IntervalManager.intervalIDs[name] = _global.setInterval(path, functionName, interval, args);
		var params:Array = new Array(path, functionName, interval);
		params = params.concat(args);
		IntervalManager.intervalIDs[name] = _global.setInterval.apply(null, params);
	}

	/**
	* Sets a function to be called at "interval" seconds after the timeout is set.
	* If there is already a timeout associated with the name that you provide
	* (meaning an interval that was created with but not cleared with the IntervalManager),
	* the IntervalManager will clear the old timeout and set the new timeout you
	* provide in it's place.
	*
	* @param 	name (String) The name you are assigning the timeout
	* @param 	path (Object) The path to the function that will be called by the setTimeout
	* @param 	functionName (String) The name of the function as a string
	* @param 	interval (Number) The interval that this function will be called on
	* @param 	args (Array) The args that will be passed to the timeout.  The arguments are passed as an array, so be sure
	*        				 to have your function compensate for that.
	* @return	Void
	*/
	public static function setTimeout(name:String, path:Object, functionName:String, interval:Number, args:Array):Void {
		if(IntervalManager.intervalIDs[name] != undefined) {
			IntervalManager.clearInterval( name );
		};
		
		IntervalManager.intervalIDs[name] = _global.setTimeout(path, functionName, interval, args);
	}

	/**
	* Clears the interval with the name you provide.
	* @param 	name (String) The name of the interval you'd like to clear.
	* @return	Void
	*/
	public static function clearInterval(name:String):Void {
		_global.clearInterval( IntervalManager.intervalIDs[name] );
		delete IntervalManager.intervalIDs[name];
	}
}
