﻿import com.fwa.events.IListenable;
import com.fwa.events.ListenerBroadcaster;
import mx.utils.Delegate;

/**
* TimeCounter class
* Provides methods to create and manage a Timer (count-up or count-down in a specific timeframe).
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.0
* @date 	23 Nov 2006
*/
class com.fwa.utils.TimeCounter implements IListenable {
	
	//broadcaster
	private var broadcaster:ListenerBroadcaster;
	
	// core
	private var end_time:Number;		// Target time in seconds 
	private var begin_time:Number;		// beginning time in seconds
	private var _delta:Number;			// Delta (in seconds) since start operation
	private var _target_delta:Number;	// number of seconds between start and begin times
	private var start_millisecs:Number;	// Start time in milliseconds - from getTimer()
	private var count_up:Boolean;		// true if counting up, false if counting down
	private var update_int:Number; 		// interval ID that executes the timer update
	private var timer_data:Object		// stores the timer information (hours, minutes, seconds, frac)
	
	/**
	* Constructor
	* Initialises the TimeCounter with the given time interval
	* Use <start> to begin the count.
	* 
	* @param	begin (Number) the time (in seconds) in which the TimeCounter wil begin
	* @param	end (Number) the time (in seconds) in which the TimeCounter will stop counting
	*/
	public function TimeCounter(begin:Number, end:Number)	{
		broadcaster = new ListenerBroadcaster()
		
		begin_time = (begin>0)? begin : 0
		end_time = (end>0)? end : 0
		
		start_millisecs = 0.0;
		count_up = true;
		_delta = 0.0;
		_target_delta = 0.0;
		
		timer_data = new Object({hours:0, minutes:0, seconds:0, frac:0})
	}
	
	/**
	* Executed on every update Interval.
	* Updates the current TimeCounter data
	* @return	Void
	*/
	private function updateTimer():Void {
		_delta = 0.001*(getTimer() - start_millisecs);
		
		// update time values
		var count:Number     = count_up ? (begin_time+_delta) : (begin_time-_delta);
		count                = Math.max(count,0);
		
		var hours:Number     = Math.floor(count*0.00027777777777777);
		var remainder:Number = count - hours*3600.0;
		var minutes:Number   = Math.floor(remainder*0.0166666666666666667);
		var fSeconds:Number  = remainder - minutes*60;
		var seconds:Number   = Math.floor(fSeconds);
		var frac:Number      = fSeconds - seconds;
		frac                 = Math.floor(10*frac);
		
		//store values into time_data object
		timer_data.hours = hours
		timer_data.minutes = minutes
		timer_data.seconds = seconds
		timer_data.frac = frac
		
		//broadcast timer update with props
		broadcaster.broadcastMessage("onUpdated",[this,timer_data])
		
		// test if target delta has been met or exceeded
		if ( _delta >= _target_delta ) {
			shutdown(_delta)
		}
	}
	
	/**
	* Returns a string with the current time data in the format [hh:mm:ss:ff]
	* @return	String
	*/
	public function toString():String {
		var stamp:String = (timer_data.hours.toString() + ":");
		if ( timer_data.minutes < 10 ) {
		  if ( timer_data.hours > 0 ) {
			stamp += ("0" + timer_data.minutes.toString() + ":");
		  }else{
			stamp += (timer_data.minutes.toString() + ":");
		  }
		}else{
		  stamp += (timer_data.minutes.toString() + ":");
		}
		
		if ( timer_data.seconds < 10 ){
		  stamp += ("0" + timer_data.seconds.toString());
		}else{
		  stamp += timer_data.seconds.toString();
		}
		stamp += "." + timer_data.frac.toString();
		return(stamp)
	}
	
	
	/**
	* Starts the time count.
	* @return 	Void
	*/
	public function start():Void {
		if ( end_time == begin_time ) {
		  shutdown()
		  return;
		}
		
		// count-up or down is determined by delta between begin and target time
		_target_delta = end_time - begin_time;
		count_up = ( _target_delta > 0 );
		if( !count_up ){
			_target_delta = - _target_delta;
		}
		
		// start timing ...
		start_millisecs = getTimer();
		clearInterval(update_int)
		update_int = setInterval(this, "updateTimer", 100)
		
		// fire 'onStart' event
		//dispatchEvent( {target:end_time, type:"onStart"} ); 
		broadcaster.broadcastMessage("onStart", [end_time])
	}
	
	public function shutdown(delta:Number):Void {
		delta = (!delta || delta==undefined)? 0 : delta
		clearInterval(update_int)
		broadcaster.broadcastMessage("onComplete",[this, delta])
	}
	
	//GETTERS-SETTERS
	/**
	* Retrieves the End Time
	* @return	Number
	*/
	public function get end():Number { return end_time; }
	
	/**
	* Sets the End Time
	* @param	_t (Number) the end time
	* @return	Void
	*/
	public function set end(_t:Number):Void {
		if ( _t < 0 ){
			broadcaster.broadcastMessage("onError", [this, "Invalid target time:"+_t.toString()])
		}else{
			end_time = _t;
		}
	}
	
	/**
	* Sets the Begin Time
	* @param	_t (Number) the begin time
	* @return	Void
	*/
	public function set begin(_t:Number):Void {
		if ( _t < 0 ){
			broadcaster.broadcastMessage("onError", [this, "Invalid beginning time: " + _t.toString()])
		}else{
			begin_time = _t;
		}
	}

	//LISTENER - BROADCASTER
	public function addListener(obj:Object):Boolean {
		return(broadcaster.addListener(obj))
	}
	public function removeListener(obj:Object):Boolean {
		return(broadcaster.removeListener(obj))
	}
}