import com.fwa.utils.Timer;
import com.fwa.data.NodeRetriever;
import com.fwa.system.FWAConfiguration;
import com.fwaapi.Accesibilidad;
import com.fwaapi.Application;

/**
* Static class Log
* Logs events in a History Object. 
* The History Obj. has an Array ("logs") inside so the length of the History can be retrieved. 
* That way you can run through the array and retrieve all logged events.
* 
* @version	1.4
* @since 	2 Dec 2006
*/

/**
* @ignore
* MODIFICATION HISTORY
* 1.1 - 3 Jan 2007
* + Added the message parsing to inject the score into the log history
* + Added the Log History SO (init - update - clear)
* 
* 1.2 - 10 Jan 2007
* + Added "onUpdate" method executed from <save> <clear> and <reset>
* 
* 1.3 - 16 Jan 2007
* + Modified the SharedObject so the log is stored per application name
* + Modified to store only 2 sessions per application
* + Added methods to validate the SharedObject's name
* + Added storage of the SO Name as a private property instead of calculating it
*   every time it is requested.
* 
* 1.4 - 17 Jan 2007
* + Modified to store N sessions per application.
*   N is obtained from mainconfig.xml
*/
class com.fwaapi.Log {
	
	private static var timer:Timer = new Timer();
	private static var history:Object;
	private static var soIndex:Number;
	private static var soName:String = null
	
	public static var debugmode:Boolean = false;
	public static var onUpdate:Function;
	
	/**
	* Constructor
	*/
	private function Log(){};
	
	/**
	* Saves a message and score in the Log's History
	* @param	identifier (String) Identifier of the message to save
	* @return	Void
	*/
	public static function save(identifier:String, accumulated_score:Number, total_score:Number):Void {
		var time:String = getTime();
		var str:String = (!debugmode)? Accesibilidad.getLabel(identifier) : identifier;
		var str_ascore:String = (accumulated_score!==undefined)? String(accumulated_score) : ""
		var str_tscore:String = (total_score!==undefined)? String(total_score) : ""
		
		str = parseScore(str, str_ascore, str_tscore);
		
		store(str, time);
		onUpdate();
	}
	
	/**
	* Retrieves the current Log's session History
	* @return	Object
	*/
	public static function retrieve():Object {
		return(history);
	}
	
	/**
	* Retrieves all the Logged events from the SharedObject
	* @return	Array
	*/
	public static function retrieveFull():Array {
		var hSO:SharedObject = getSO();
		return(hSO.data.events)
	}
	
	/**
	* Resets the Log's History
	* @return	Void
	*/
	public static function reset():Void {
		history = new Object({logs:[]});
		soName = null
		initHistorySO();
		timer.reset();
		onUpdate();
	}
	
	/**
	* Clears all previous History Events in the Log keeping only the ones 
	* from the last Session.
	* @return	Void
	*/
	public static function clear():Void {
		var hSO:SharedObject = getSO();
		hSO.data.events = undefined;
		hSO.flush();
		initHistorySO();
		updateHistorySO();
		onUpdate();
	}
	
	/**
	* Retrieves a string with the formatted elapsed time
	* @return	String
	*/
	public static function getTime():String {
		var elapsed:Object = timer.getElapsedObject();
		
		var h:String = (elapsed.hours<10)? "0"+elapsed.hours : elapsed.hours;
		var m:String = (elapsed.minutes<10)? "0"+elapsed.minutes : elapsed.minutes;
		var s:String = (elapsed.seconds<10)? "0"+elapsed.seconds : elapsed.seconds;
		
		var parsed:String = h+":"+m+":"+s;
		return(parsed);
	}
	
	
	/**
	* Creates a new node in the Log's History with the given message and time
	* @param	message (String) message to store
	* @param	time (String) time to store
	* @return	Void
	*/
	private static function store(message:String, time:String):Void {
		if(history == undefined){
			reset();
			updateAfterEvent();
		}
		var log:Object = new Object({message:message, time:time});
		history.logs.push(log);
		updateHistorySO();
	}
	
	/**
	* Parses a Log message and injects the provided score values into the message fields.
	* If the Log message has no fields then the message will output as is.
	* Returns a String with the parsed message.
	* 
	* @param	msg (String) Log message
	* @param	a_score (String) Accumulated Score value
	* @param	t_score (String) Total Score value
	* @return	String
	*/
	private static function parseScore(msg:String, a_score:String, t_score:String):String {
		var str_ascore:String = "PARTIAL_SCORE";
		var str_tscore:String = "TOTAL_SCORE";
		
		//if the String has the Accumulated Score delimiter, replace
		if(msg.indexOf(str_ascore)!==-1){
			msg = msg.split(str_ascore).join(a_score);
		}
		
		//if the String has the Total Score delimiter, replace
		if(msg.indexOf(str_tscore)!==-1){
			msg = msg.split(str_tscore).join(t_score);
		}
		
		return(msg)
	}
	
	/**
	* Initialises the history SharedObject
	* @return 	Void
	*/
	private static function initHistorySO():Void {
		//get SO reference
		var hSO:SharedObject = getSO();
		
		//if no History exists, create the Array
		if(hSO.data.events == undefined){
			hSO.data.events = new Array();
		}
		var soEvents:Array = hSO.data.events;
		
		//create a new history event at the beginning of the current events array
		var today_date:Date = new Date();
		var date_str:String = (today_date.getDate()+"/"+(today_date.getMonth()+1)+"/"+today_date.getFullYear());
		var time_str:String = today_date.getHours()+":"+today_date.getMinutes()+":"+today_date.getSeconds();
		var eventName:String = "Session: "+date_str+" - "+time_str;
		var eventHistory:Array = new Array();
		soEvents.unshift({name:eventName, history:eventHistory});
		
		//keep only the amount of sessions configured in the mainconfig.xml file
		var maxEvents:Number = Number(FWAConfiguration.main.logs.max_history._value);
		var eventsCount:Number = soEvents.length;
		if(eventsCount>maxEvents){
			var deleteCount:Number = eventsCount - maxEvents;
			soEvents.splice(maxEvents, deleteCount);
		}
		hSO.flush();
	}
	
	/**
	* Updates the latest Event's history with the current history logs
	* @return	Void
	*/
	private static function updateHistorySO():Void {
		var hSO:SharedObject = getSO();
		var soEvents:Array = hSO.data.events;
		
		soEvents[0].history = history.logs;
		hSO.flush();
	}
	
	/**
	* Returns a reference to the current application's Shared Object
	* @return	SharedObject
	*/
	private static function getSO():SharedObject {
		if(soName == null){
			soName = convertToFileName(Application.getApplicationName());
			trace("SO NAME:"+soName);
		}
		return(SharedObject.getLocal(soName));
	}
	
	
	/**
	* Converts the given string into a valid filename format String.
	* @param	str (String) String to sanitize
	* @return	String
	*/
	private static function convertToFileName(str:String):String {
		var chars:Number = str.length;
		var output:String = ""
		
		for (var i:Number = 0; i<chars; i++){
			var char:String = str.substr(i, 1);
			output+= isValidChar(char)? char : "_"
		}
		
		return(output);
	}
	
	/**
	* Loops through an array of valid ascii char codes and returns true 
	* if the given char is between that range. Otherwise returns false.
	* @param	char (String) Char to evaluate
	* @return	Boolean
	*/
	private static function isValidChar(char:String):Boolean {
		var validRanges:Array = new Array([45,46],[48,57],[65,90],[95,95],[97,122])
		var ranges:Number = validRanges.length;
		var charCode:Number = char.charCodeAt(0);
		
		for (var i:Number = 0; i<ranges; i++){
			var range_min:Number = validRanges[i][0];
			var range_max:Number = validRanges[i][1];
			if (charCode>=range_min && charCode<=range_max){
				return(true);
			}
		}
		
		return(false);
	}
	
}