import com.fwa.system.FWAConfiguration;
import com.fwa.loaders.SoundLoader;
import com.fwa.data.NodeRetriever;
import com.fwa.data.ObservableSound;
import com.fwa.events.IListenable;
import com.fwa.events.ListenerBroadcaster;
import com.fwa.debug.Tracer;
import mx.utils.Delegate;

/**
* This static class works as an API to preload an execute sounds in the Framework, by implementing the merge between the 
* FWA SoundLoader, ObservableSound, Configuration, and several components.
* 
* @version	1.3
* @since 	21 Nov 2006
*/
class com.fwaapi.Sounds {
	//properties
	private static var main_volume:Number = 100;
	private static var playlist:Array = new Array();
	
	//callbacks
	public static var onPreloadStart:Function;
	public static var onPreloadProgress:Function;
	public static var onPreloadComplete:Function;
	
	/**
	* Constructor
	*/
	private function Sounds(){}
	
	/**
	* Initialises the playing of the given identifier's sound
	* @param	identifier (String) Identifier of the sound to play
	* @param	endFunc (Function) Callback to execute when the sound is completed
	* @param	endArgs	(Array) List of parameters to pass to the Callback
	* @param 	endScope (Object) Scope where the Callback is executed
	* @return	Void
	*/
	public static function playSound(identifier:String, endFunc:Function, endArgs:Array, endScope:Object):Void {
		endScope = (endScope == undefined)? null : endScope
		endArgs = (endArgs == undefined)? null : endArgs
		
		var sNode:Object = getNode(identifier);
		if(sNode!==undefined){
			//if there's no soundObject stored for that identifier create one
			//in that case, the sound will not be preloaded.. will play as it finishes loading
			
			//create the onSoundComplete Listener
			var sObj_lst:Object = new Object();
			sObj_lst.onSoundComplete = function(snd:ObservableSound){
				endFunc.apply(endScope,endArgs);
				Sounds.removeFromPlaylist(identifier)
			}
			
			if(sNode.soundObject == undefined){
				var snd_filename:String = getSoundsPath() + sNode.file._value
				var sObj:ObservableSound = new ObservableSound();
				
				sObj_lst.onLoadComplete = function(snd:ObservableSound) {
					snd.setVolume(Sounds.getVolume());
					snd.start();
				};
				sObj.addListener(sObj_lst);
				
				sObj.loadSound(snd_filename);
				//sObj.setVolume(getVolume());
				sNode.soundObject = sObj;
			}else{
				//Tracer.output("Playing cached sound:"+sNode.soundObject.toString(), "playSound", "Sounds", "com.fwaapi")
				sNode.soundObject.setVolume(getVolume());
				sNode.soundObject.addListener(sObj_lst)
				sNode.soundObject.start();
			}
			
			addToPlaylist(sNode.soundObject, identifier)
		}else{
			Tracer.output("Sound doesn't exists!", "playSound", "Sounds", "com.fwaapi")
		}
	}
	
	
	/**
	* Creates a preload job with all the sounds passed in the identifiers array.
	* After the preload job is completed, the endFunction is called in the given scope, with the given params.
	* 
	* @param	identifiers (Array) List of sound identifiers to preload
	* @param	endFunc (Function) Function to invoke when all identifiers are preloaded
	* @param	endArgs (Array) Arguments to pass to the end function
	* @param	endScope (Object) Scope of the end function
	* @return	Void
	*/
	public static function preloadSounds(identifiers:Array, endFunc:Function, endArgs:Array, endScope:Object):Void {
		endScope = (endScope == undefined)? null : endScope
		endArgs = (endArgs == undefined)? null : endArgs
		
		var sl:SoundLoader = new SoundLoader()
		var count:Number = identifiers.length;
		
		//retrieve all identifiers and push the files into the SoundLoader
		for(var i:Number = 0; i<count; i++){
			var sNode:Object = NodeRetriever.retrieveFirstNode(identifiers[i], FWAConfiguration.sounds.sound)
			//trace("sNode:"+sNode.file._value)
			//if the node exists and doesn't have a Sound already cached
			if(sNode!==undefined && sNode.soundObject == undefined){
				var s_file:String = getSoundsPath() + sNode.file._value
				sNode.soundObject = new ObservableSound()
				//sNode.soundObject.setVolume(getVolume())
				var added:Boolean = sl.addSound(s_file, sNode.soundObject)
				//Tracer.output("sound preloaded ok:"+added, "preloadSounds", "Sounds", "com.fwaapi")
			}
		}
		
		var sl_lst:Object = new Object();
		sl_lst.onProcessComplete = function(){
			Sounds.onPreloadComplete()
			endFunc.apply(endScope, endArgs)
		}
		sl_lst.onProcessProgress = Delegate.create(Sounds, Sounds.onPreloadProgress)
		sl_lst.onProcessStart = Delegate.create(Sounds, Sounds.onPreloadStart)
		sl.addListener(sl_lst)
		sl.process()
	}
	
	
	/**
	* Sets the volume for all the sounds
	* @param	val (Number) Volume to set
	* @return	Void
	*/
	public static function setVolume(val:Number):Void {
		main_volume = val
		updateAllVolumes()
	}
	
	/**
	* Returns the main volume for all sounds
	* @return 	Number
	*/
	public static function getVolume():Number {
		return main_volume
	}
	
	
	/**
	* Stops and removes all the sounds in the current list
	* @return	Void
	*/
	public static function removeAll():Void {
		var count:Number = playlist.length;
		for(var i:Number = 0; i<count; i++){
			var snd:ObservableSound = playlist[i].sound;
			var id:String = playlist[i].identifier;
			snd.stop();
			removeFromPlaylist(id);
		}
	}
	
	
	//PRIVATE METHODS
	/**
	* Adds a sound to the playlist
	* @param	snd (ObservableSound) sound to add
	* @param	id (String) identifier of the added sound
	* @return	Void
	*/
	private static function addToPlaylist(snd:ObservableSound, id:String):Void {
		//Tracer.output("adding: "+id, "addToPlaylist", "Sounds", "com.fwaapi")
		var obj:Object = {sound:snd, identifier:id};
		playlist.push(obj)
	}
	
	/**
	* Removes a sound from the playlist
	* @param	id (String) identifier of the sound to remove
	* @return	Void
	*/
	private static function removeFromPlaylist(id:String):Void {
		//Tracer.output("removing: "+id, "removeFromPlaylist", "Sounds", "com.fwaapi")

		var removed:Boolean = false;
		for(var i:Number=0; i<playlist.length; i++){
			if(playlist[i].identifier == id){
				playlist.splice(i,1)
				removed = true
			}
		}
		//Tracer.output("removed:"+removed, "removeFromPlaylist", "Sounds", "com.fwaapi");
	}
	
	/**
	* Updates the volume of all sounds in the playlist
	* @return	Void
	*/
	private static function updateAllVolumes():Void {
		//Tracer.output("updating volumes..", "updateAllVolumes", "Sounds", "com.fwaapi")
		for(var i:Number=0; i<playlist.length; i++){
			var snd:ObservableSound = ObservableSound(playlist[i].sound);
			snd.setVolume(main_volume);
			//Tracer.output("updated:"+playlist[i].identifier+" to:"+main_volume, "updateAllVolumes", "Sounds", "com.fwaapi")
		}
	}
	
	
	/**
	* Retrieves the full path to the sounds folder.
	* @return	String
	*/
	private static function getSoundsPath():String {
		var app_path:String = FWAConfiguration.application+FWAConfiguration.language
		var sounds_path:String = app_path+"/sounds/"
		return(sounds_path)
	}
	
	/**
	* Retrieves the given identifier's node.
	* @param	identifier (String) Identifier of the node that wants to be retrieved
	* @return	Object
	*/
	private static function getNode(identifier:String):Object {
		var obj:Object = FWAConfiguration.sounds
		var node:Object = NodeRetriever.retrieveFirstNode(identifier, obj.sound)
		
		return( node )
	}
}