import com.fwa.events.IListenable;
import com.fwa.events.ListenerBroadcaster;
import mx.utils.Delegate;

/**
* ObservableSound class
* This class extends the native Sound Class and adds the ability to broadcast events, so you can add multiple listeners.
* It also implements onLoadProgress and onLoadBegin messages to allow a more accurate preload of the sound.
* The original <onLoad> <onSoundComplete> and <onID3> were replaced by messages broadcasted.
* 
* Broadcasts:
* 
* Message: 
* <onLoadStart> When the ObservableSound begins to load
* Params:
* <li>osnd (ObservableSound) The current instance of the ObservableSound</li>
* 
* Message:
* <onLoadProgress> As the ObservableSound is loading the file
* Params:
* <li>osnd (ObservableSound) The current instance of the ObservableSound</li>
* <li>bl (Number) The total bytes loaded</li>
* <li>bt (Number) The total bytes of the sound</li>
* 
* Message:
* <onLoadComplete> When the ObservableSound has finished loading
* Params:
* <li>osnd (ObservableSound) The current instance of the ObservableSound</li>
* <li>success (Boolean) True if the sound loaded properly, otherwise false</li>
* <li>duration (Number) The duration of the loaded sound</li>
* 
* Message:
* <onSoundComplete> When the ObservableSound has ended playing
* Params:
* <li>osnd (ObservableSound) The current instance of the ObservableSound</li>
* 
* Message:
* <onID3> When the ID3 metadata of the file is loaded
* Params:
* <li>id3 (Object) The ID3 Object</li>
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.0
* @date 	20 Nov 2006
*/
class com.fwa.data.ObservableSound extends Sound implements IListenable {
	
	private var broadcaster:ListenerBroadcaster;
	private var progress_int:Number;
	private var soundfile:String;
	
	/**
	* Creates a new ObservableSound object for a specified movie clip. 
	* If you do not specify a target instance, the Sound object controls all of the sounds in the movie.
	* @param	target (Object) The movie clip instance on which the Sound object operates.
	*/
	public function ObservableSound(target:Object){
		super(target)
		broadcaster = new ListenerBroadcaster()
	}
	
	/**
	* Loads an MP3 file into a Sound object. 
	* You can use the isStreaming parameter to indicate whether the sound is an event or a streaming sound. 
	* 
	* Event sounds are completely loaded before they play. 
	* They are managed by the ActionScript Sound class and respond to all methods and properties of this class.
	* 
	* Streaming sounds play while they are downloading. 
	* Playback begins when sufficient data has been received to start the decompressor. 
	* @param	url (String) The location on a server of an MP3 sound file.
	* @param	isStreaming (Boolean) A Boolean value that indicates whether the sound is a streaming sound (true) or an event sound (false).
	* @return	Void
	*/
	public function loadSound(url:String, isStreaming:Boolean):Void {
		soundfile = url
		super.loadSound(url, isStreaming)
		broadcaster.broadcastMessage("onLoadStart", [this, url])
		
		//create an interval to poll for the sound load progress
		clearInterval(progress_int)
		progress_int = setInterval(this, "broadcastProgress", 50)
	}
	
	/**
	* @return	String
	*/
	public function toString():String {
		return(soundfile)
	}
	
	/**
	* Broadcasts the onLoadProgress message.
	* @return	Void
	*/
	private function broadcastProgress():Void {
		var bL:Number = super.getBytesLoaded()
		var bT:Number = super.getBytesTotal()
		broadcaster.broadcastMessage("onLoadProgress",[this, bL, bT])
	}
	
	
	/**
	* Overrides the Sound.onLoad callback by broadcasting the onLoadComplete message.
	* @param	success (Boolean) True if the sound loaded properly
	* @return	Void
	*/
	private function onLoad(success:Boolean):Void {
		broadcaster.broadcastMessage("onLoadComplete", [this, success, super.duration])
		//clear the load progress interval
		clearInterval(progress_int)
	}
	
	/**
	* Overrides the Sound.onID3 callback by broadcasting the onID3 message.
	* @return	Void
	*/
	private function onID3():Void {
		broadcaster.broadcastMessage("onID3", [super.id3])
	}
	
	/**
	* Overrides the Sound.onSoundComplete callback by broadcasting the onSoundComplete message. 
	*/
	private function onSoundComplete():Void {
		broadcaster.broadcastMessage("onSoundComplete", [this])
	}
	
	
	//listener-broadcaster integration
	/**
	* @see	com.fwa.events.ListenerBroadcaster
	*/
	public function addListener(obj:Object):Boolean {
		return(broadcaster.addListener(obj))
	}
	
	/**
	* @see	com.fwa.events.ListenerBroadcaster
	*/
	public function removeListener(obj:Object):Boolean {
		return(broadcaster.removeListener(obj))
	}
}