﻿import com.fwa.events.ListenerBroadcaster;
import com.fwa.events.IListenable;
import mx.utils.Delegate;

/**
* FilesLoader class
* Creates a process to load files (SWFs or flash supported images) sequentially
* * Broadcasts:
* 
* Message: 
* <onProcessStart> When the FilesLoader Queue begins to load the first item
* Params:
* <li>items (Number) The total of items to load
* 
* Message: 
* <onProcessProgress> As the Queue processes each item. Sends the overall progress of the queue, and the load progress of the current item 
* Params:
* <li>overall_percent (Number) The progress percent of the overall queue
* <li>item_percent (Number) The progress percent of the current loading item
* 
* Message: 
* <onProcessComplete> When the last item of the Queue has finished loading
* Params:
* <li>none</li>
* 
* Message: 
* <onProcessAborted> When the <abort> method is invoked and all remianing preload processes has been shutdown
* Params:
* <li>none</li>
* 
* Message: 
* <onProcessError> When trying to process an queue that is paused or is not idle
* Params:
* <li>none</li>
* 
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	2.0
* @date 	28 nov 2005
*/
class com.fwa.loaders.FilesLoader implements IListenable {
	
	//properties
	private var idle:Boolean
	private var paused:Boolean
	
	private var clip_dummy:MovieClip
	private var clip_loader:MovieClipLoader
	private var clip_listener:Object
	
	private var elements:Array; //stores a list of elements to preload
	private var elements_total:Number; //stores the total aomunt of elements at the beginning of the process
	
	private var broadcaster:ListenerBroadcaster
	
	
	/**
	* Constructor
	* @param	dummy (MovieClip) target clip that will be used as dummy to preload content
	*/
	public function FilesLoader(dummy:MovieClip){
		elements = new Array()
		clip_dummy = (dummy==undefined)? createDummyClip() : dummy
		idle = true
		paused = false
		clip_loader = new MovieClipLoader()
		//createLoadListener() //listener que retransmite los eventos onLoad, onInit, onProgress de los MCs
		clip_listener = getListener()
		broadcaster = new ListenerBroadcaster() //broadcaster interno de la clase
	}
	
	
	/**
	* Adds a file to the FilesLoader's list.
	* @param	filename (String) File to load
	* @param 	listener (Object) Object to notify with load events
	* @param 	holder (MovieClip) Target clip to load filename
	* @return	Void
	*/
	public function addFile(filename:String, listener:Object, holder:MovieClip):Void {
		//if no holder provided to perform load, use the Dummy
		holder = (holder==undefined)? this.clip_dummy : holder
		
		//create Object to pass to Collection
		var obj:Object = new Object({filename:filename, holder:holder, listener:listener})
		
		//send object to list
		elements.push(obj)
		elements_total = elements.length
	}
	
	
	/**
	* Empties de ProgressiveLoader's queue.
	* @return	Void
	*/
	public function clear():Void {
		pause()
		elements = new Array()
	}
	
	
	/**
	* Pauses/Resumes the ProgressiveLoader queue.
	* @param 	resume (Boolean= If true, resumes the queue process (if previously paused)
	* @return	Void
	*/
	public function pause(resume:Boolean):Void {
		if(resume == true){
			if(paused == true){
				paused = false
				process()
			}
		}else{
			if(paused==false){
				paused = true
				//pauseQueue()
			}
		}
	}
	
	
	/**
	* Starts loading the next element of the queue.
	* Sets the ProgressiveLoader's *idle* status to *true*
	* @return	Void
	*/
	public function process():Void {
		//if loader is Idle, and there are elements left in the Collection, process
		if(idle==true && paused==false && elements.length>0){
			//if next element is a valid element (has holder, and has filename)
			var elem:Object = elements[0]
			if(elem.filename!==undefined && elem.filename!=="" && elem.holder!==undefined){
				//perform load of next Element
				attachListener()
				clip_loader.loadClip(elem.filename, elem.holder)
				idle = false
			}
		}else{
			//in case Queue can't be processed
			if(elements.length==0){
				//empty queue 
				broadcaster.broadcastMessage("onProcessComplete")
				elements_total = 0
			}else{
				broadcaster.broadcastMessage("onProcessError")
			}
		}
	}
	
	
	/**
	* Aborts the current and remaining file's load process.
	* @return	Void
	*/
	private function abort():Void {
		clip_loader.unloadClip(elements[0].holder)
		deattachListener()
		idle = true
		broadcaster.broadcastMessage("onProcessAborted")
	}
	

	/**
	* Removes the currently processed Queue's element.
	* @return	Void
	*/
	private function removeLastProcessedElement():Void {
		elements.shift()
	}
	

	/**
	* Sends a message to the current element's Listener.
	* @param 	msg (String) String with the message to execute
	* @param 	parameters (Array) Array of parameters to pass to the Listener
	* @return	Void
	*/
	private function broadcastMessageToElement(msg:String, parameters:Array):Void {
		var elem:Object = elements[0]
		elem.listener[msg].apply(null,parameters)
	}
	
	
	/**
	* Creates a dummy clip and returns its reference
	* @return	MovieClip
	*/
	private function createDummyClip():MovieClip {
		var cdummy:MovieClip = (!_root.__dummy)? _root.createEmptyMovieClip("__dummy",_root.getNextHighestDepth()) : _root.__dummy
		cdummy._visible = false
		return (cdummy)
	}
	
	
	/**
	* Attachs the ProgressiveLoader's main listener to the current element's holder clip
	* This Listener re-broadcast to the Element's listener events, and also executes the
	* *process* method when it finishes loading the image.
	* @return	Void
	*/
	private function attachListener():Void {
		clip_loader.addListener(clip_listener)
	}
	
	
	/**
	* Deattachs the ProgressiveLoader's main listener from the current element's holder clip
	* @return	Void
	*/
	private function deattachListener():Void {
		clip_loader.removeListener(clip_listener)
	}
	
	
	/**
	* Registers the "onLoadStart" message broadcasted by the current loading item.
	* Broadcasts onProcessStart
	* @param	snd (ObservableSound) the sound being loaded
	* @return	Void
	*/
	private function onItemLoadStart(tmc:MovieClip):Void {
		if(elements.length == elements_total){
			broadcaster.broadcastMessage("onProcessStart", [elements_total])
		}
		tmc.stop()
		tmc._visible = false
	}
	
	/**
	* Registers the "onLoadProgress" message broadcasted by the current loading item.
	* Calculates the overall progress of the Queue and broadcasts the "onProcessProgress" message
	* @param	snd (ObservableSound) the sound being loaded
	* @param	bL (Number) the total bytes loaded
	* @param	bT (Number) the total bytes
	* @return	Void
	*/
	private function onItemLoadProgress(tmc:MovieClip, bL:Number, bT:Number):Void {
		var current_item:Number = (elements_total+1) - elements.length
		var current_item_percent:Number = Math.ceil(bL * 100 / bT)
		var single_item_percent:Number = Math.ceil(100 / elements_total)
		var current_item_overall_percent:Number = single_item_percent * current_item_percent / 100
		var previous_items_overall_percent:Number = (current_item-1) * single_item_percent
		var overall_percent:Number = Math.ceil(previous_items_overall_percent + current_item_overall_percent)
		
		overall_percent = (overall_percent>100)? 100 : overall_percent;
		
		broadcaster.broadcastMessage("onProcessProgress", [overall_percent, current_item_percent])
	}
	
	/**
	* Registers the "onLoadComplete" message broadcasted by the current loading item.
	* Proceeds to the next item in the queue
	* @param	snd (ObservableSound) the sound being loaded
	* @param	success (Boolean) true if the sound loaded properly
	* @param	duration (Number) the sound's duration
	* @return	Void
	*/
	private function onItemLoadComplete(tmc:MovieClip):Void {
		idle = true
		removeLastProcessedElement()
		process()
	}
	
	/**
	* Returns a Listener Object for the file being loaded
	* @return	Object
	*/
	private function getListener():Object {
		var lst:Object = new Object();
		lst.onLoadStart = Delegate.create(this, onItemLoadStart)
		lst.onLoadProgress = Delegate.create(this, onItemLoadProgress)
		lst.onLoadComplete = Delegate.create(this, onItemLoadComplete)
		lst.onLoadError = Delegate.create(this, onItemLoadComplete)
		return(lst)
	}
	
	//LISTENER BROADCASTER INTEGRATION///////////////////////////////////////////////////////////
	/**
	* Adds an object to the events notification list
	* @param	obj (Object) object to be added
	* @return	Boolean
	*/
	public function addListener(obj:Object):Boolean{
		return (broadcaster.addListener(obj))
	}
	
	/**
	* Removes an object from the events notification list
	* @param	obj (Object) object to be removed
	* @return	Boolean
	*/
	public function removeListener(obj:Object):Boolean{
		return (broadcaster.removeListener(obj))
	}

}