import com.wunderman.debug.Tracer;
import com.wunderman.ui.Windows;
import com.wunderman.ui.Window;

/**
* @class 	WindowsProxy
* @tooltip 	This class provides methods to manage specific types of windows (alerts, popups, etc)
* 			serving as a proxy to com.wunderman.ui.Windows
* @author 	Federico Campo Piombi
* @version 	1.0
* @date 	01 Nov 2006
* 
* @see 		com.wunderman.ui.Placeholders
* @see 		com.wunderman.ui.Windows
* @see		com.wunderman.ui.Window
*/
class com.parker.content.WindowsProxy {
// properties:
	
	/**
	* @property _instance
	* @tooltip 	Stores the Singleton instance
	* @type 	WindowsManager
	*/
	private static var _instance:WindowsProxy = null;
	
	private var content_ph:String
	private var popups_ph:String
	private var alerts_ph:String
	
	private var windows:Windows

	
//CONSTRUCTOR: 
	private function WindowsProxy() {
		windows = Windows.getInstance()
		content_ph = "__CONTENT"
		popups_ph = "__POPUPS"
		alerts_ph = "__ALERTS"
	}

	
// PUBLIC methods:

	/**
	* @method 	getInstance
	* @tooltip 	Resturns a reference to the Singleton WindowsManager class
	* @return 	WindowsManager
	*/
	public static function getInstance():WindowsProxy {
		if(WindowsProxy._instance == null){
			WindowsProxy._instance = new WindowsProxy()
		}
		return(WindowsProxy._instance)
	}
	
	/**
	* @method 	openAlert
	* @tooltip 	Opens a new Alert Window and returns its reference
	* @param 	title (String) Alert Title
	* @param 	message (String) Alert Message
	* @param 	mode (String) Alert window type (alert, registrologin, etc)
	* @param 	callback (Object) Object with callback functions
	* @return 	Window
	*/
	public function openAlert(title:String, message:String, mode:String, callback:Object):Window {
		//requires __ALERTS Placeholder (uses modal clip already in alert_mc)
		debug("openAlert","Title: "+title+" Message:"+message)
		var wnd:Window = windows.open(alerts_ph, {identifier:"single_alert", linkage:"clip_messagebox", modal:true})
		var clip_alert:MovieClip = wnd.getContentClip()
		clip_alert.setAlert(title, message, mode, callback)
		clip_alert._x = Math.round((Stage.width/2) - (clip_alert._width/2))
		clip_alert._y = Math.round((Stage.height/2) - (clip_alert._height/2))
		debug("openAlert","alert clip:"+clip_alert._name)
		return(wnd)
	}

	/**
	* @method 	openPopup
	* @tooltip 	Opens a new Popup Window and returns its reference
	* @param 	p_filename (String) file to load in the popup
	* @param 	p_modal (Boolean) wether the popup is modal or not
	* @return 	Window
	*/
	public function openPopup(p_filename:String, p_modal:Boolean):Window {
		//requires __POPUPS Placeholder
		var wnd_identifier:String = windows.retrieveWindowIdentifier(p_filename)
		debug("openPopup", "filename:"+p_filename+" modal:"+p_modal+" identifier:"+wnd_identifier)
		var wnd:Window = windows.open(popups_ph, {identifier:wnd_identifier, filename:p_filename, modal:p_modal})
		return(wnd)
	}

	/**
	* @method 	openWindow
	* @tooltip 	Opens a new Content Window and returns its reference
	* @param 	content (String) external file or identifier to load in the Window
	* @param 	configuration (Object) {modal:Boolean, xpos:Number, ypos:Number}
	* @return 	Window
	*/
	public function openWindow(content:String, config:Object):Window {
		//requires __WINDOWS Placeholder
		windows.closeAllFromPlaceholder(content_ph)
		var wnd_identifier:String = windows.retrieveWindowIdentifier(content)
		var wnd:Window = windows.open(content_ph, {identifier:wnd_identifier, filename:content, modal:(config.modal==undefined)? false : config.modal})
		wnd._x = (config._x)? config._x : 0
		wnd._y = (config._y)? config._y : 0
		if(config.display && config.display == false){
			wnd.hide()
		}
		return(wnd)
	}
	
	/**
	* @method	attachWindow
	* @tooltip 	Attaches a library clip into the Content Placeholder
	* 			To remove a window attached with attachWindow, use closeWindow (or window.close())
	* @param	linkage (String) Linkage name of the clip to attach
	* @param	modal (Boolean) True if the attached clip must have a Modal Background. Default is False
	* @return	Window
	*/
	public function attachContent(linkage:String, placeholder:String, modal:Boolean):Window {
		placeholder = (placeholder == undefined)? content_ph : placeholder
		modal = (modal!==undefined)? modal : false
		var wnd:Window = windows.open(placeholder, {identifier:linkage, linkage:linkage, modal:modal})
		return(wnd)
	}
	
	/**
	* @method 	loadBackground
	* @param	bg (String) Filename with the background to load
	* @return	Window
	*/
	public function loadBackground(bg:String):Window {
		windows.closeAllFromPlaceholder("__BACKGROUND")
		var wnd:Window = windows.open("__BACKGROUND", {filename:bg, modal:false, identifier:"bg"})
		return(wnd)
	}
	
	
	/**
	* @method closeAlert
	* @return Boolean
	*/
	public function closeAlert():Boolean {
		var alert_wnd:Window = windows.getWindow(alerts_ph,"single_alert")
		if(alert_wnd!==undefined){
			return( windows.close(alerts_ph, alert_wnd) )
		}else{
			debug("closeAlert", "Alert Window unavailable")
			return(false)
		}
	}

	/**
	* @method 	closePopup
	* @param 	wnd (Window) window to close and unregister from the DepthManager
	* @return 	Boolean
	*/
	public function closePopup(wnd:Window):Boolean {
		return( windows.close(popups_ph, wnd) )
	}

	/**
	* @method 	closeWindow
	* @param 	wnd (Window) window to close and unregister from the DepthManager
	* @return 	Boolean
	*/
	public function closeWindow(wnd:Window):Boolean {
		if(wnd == undefined){
			debug("closeWindow", "Wrong parameter 'wnd'")
			return(false)
		}else{
			return( windows.close(content_ph, wnd) )
		}
	}
	
	
	/**
	* @method 	debug
	* @tooltip 	Traces a message to the output panel
	* @param	method (Strign) Method that invokes <debug>
	* @param	msg (String) message to output
	*/
	private function debug(method:String, error:String){
		Tracer.output(error,method,"WindowsProxy")
	}

	
// listener broadcaster integration:
	/**
	* @method 	addListener
	* @tooltip 	adds a new listener to the Broadcaster's collection
	* @param 	listener_obj (Object) object to add to the Collection
	* @return 	Boolean
	*/
	public function addListener(listener_obj:Object):Boolean {
		return( windows.addListener(listener_obj) )
	}

	/**
	* @method 	removeListener
	* @tooltip 	removes a listener from the Broadcaster's collection
	* @param 	listener_obj (Object) listener to be removed
	* @return 	Boolean
	*/
	public function removeListener(listener_obj:Object):Boolean {
		return( windows.removeListener(listener_obj) )
	}

}
// ** END WindowsProxy ***************************************************