import com.wunderman.utils.Validate

/**
* @class 	FormsProcessor
* @tooltip 	This class provides methods to perform validation of TextFields, Combos and Checkboxes
* 			Added Combos and Checkboxes must be MovieClips, and need to have a getValue() method. 
* 			In case of combos, getValue will return any type of value, but for Checks the returned value
* 			must be a Boolean
* @author	Federico Campo Piombi
* @version	1.1
* @date 	01 Oct 2006
*/

class com.wunderman.forms.FormsProcessor {

//PROPERTIES
	private var fields:Array
	private var combos:Array
	private var checks:Array
	private var errors:Array
	private var default_error_message:String
	
	
//CONSTRUCTOR
	/**
	* @constructor
	*/
	public function FormsProcessor(){
		fields = new Array();
		combos = new Array();
		checks = new Array();
		errors = new Array();
		setDefaultErrorMessage("Existen campos incompletos o equivocados\nRevise la informacin suministrada")
	}
	
//PUBLIC METHODS
	/**
	* @method 	addField
	* @tooltip 	Adds a TextField (and its validation params) to the TF's validation Array
	* @param	field (TextField) TextField to validate
	* @param	type (String) Type of content to validate: text, number or email
	* @param	label (String) Error message in case validation fails for that TF
	* @param	restrict (String) Characters to restrict into that TF
	* @param	maxchars (Number) Max amount of chars to allow into that TF
	* @param	optional (Boolean) TRUE if this TF must skip the validation process
	* @return	Void
	*/
	public function addField(field:TextField, type:String, label:String, restrict:String, maxchars:Number, optional:Boolean):Void {
		// [0] = field, [1] = type, [2] = label, [3] = restrict, [4] = maxlength
		// [5] = optional. En realidad es para evitar el loop de validacion gral
		fields.push([field, type, label, restrict, maxchars, ((optional==true)? true : false)]);
		field.restrict = restrict
		field.maxChars = maxchars
		field.tabIndex = 10+fields.length
	}
	
	/**
	* @method 	addCombo
	* @tooltip 	Adds a ComboBox (and its validation params) to the CB's validation Array
	* @param	combo (MovieClip) Combobox reference
	* @param	error_value (Object) The value that will throw a validation error for this combo
	* @param	error_str (String) Error message to store if this combo fails validation
	* @param	optional (Boolean) TRUE if this combo must skip the validation process
	* @return	Void
	*/
	public function addCombo(combo:MovieClip, error_value:Object, error_str:String, optional:Boolean):Void {
		combos.push([combo, error_value, error_str, ((optional==true)? true : false)])
	}
	
	/**
	* @method 	addCheckbox
	* @tooltip 	Adds a Checkbox (and its validation params) to the CBX's validation Array
	* @param	checkbox (MovieClip) Checkbox reference
	* @param	mandatory_value (Object) The value this CBX must have for not throwing a validation error
	* @param	error_str (String) Error message to store if this checkbox fails validation
	* @param	optional (Boolean) TRUE if this checkbox must skip the validation process
	* @return 	Void
	*/
	public function addCheckbox(checkbox:MovieClip, mandatory_value:Object, error_str:String, optional:Boolean):Void {
		checks.push([checkbox, mandatory_value, error_str, ((optional==true)? true : false)])
	}
	
	/**
	* @method 	isValid
	* @tooltip	Performs the validation process for TFs, CBs and CBXs, and returns TRUE if there were no errors
	* @return	Boolean
	*/
	public function isValid():Boolean {
		// Vacia los errores
		errors = new Array();
		validateFields();
		validateCombos();
		validateChecks();
		
		// Return error value
		return (errors.length > 0) ? false : true;
	}
	
	/**
	* @method 	getError
	* @tooltip 	Returns the first error generated in the validation process
	* 			The returned object contains:
	* 			error (String) The message set for that error
	* 			field (Object) A reference to the field that threw the error (MovieClip or TextField)
	* 			type  (String) The field's validation type (combo, checkbox, text, number, email)
	* 			name  (String) For CBs and CBXs, their MovieClip names, and for TextFields the <label> param
	* 							of <addField>
	* 			
	* @return	Object
	*/
	public function getError():Object {
		return(errors[0])
	}
	
	/**
	* @method 	getErrors
	* @tooltip 	Returns the whole errors Array
	* @return	Array
	*/
	public function getErrors():Array {
		return(errors)
	}
	
	/**
	* @method 	setDefaultErrorMessage
	* @tooltip 	Sets a message to display as default for fields that has no specific error message.
	* @param	msg (String) Message to display
	* @return	Void
	*/
	public function setDefaultErrorMessage(msg:String):Void {
		default_error_message = msg
	}
	
	/**
	* @method 	clearFields
	* @tooltip 	Clears the containing text for all registered TextFields
	* @return	Void
	*/
	public function clearFields():Void {
		var fcount:Number = fields.length
		for(var i:Number =0; i<fcount; i++){
			fields[i][0].text = ""
		}
	}
	
//PRIVATE METHODS
	/**
	* @method 	validateFields
	* @tooltip 	Performs validation process for TextFields
	* @return	Void
	*/
	private function validateFields():Void {
		var fcount:Number= fields.length
		for(var i:Number =0; i<fcount; i++) {
			var field:TextField = fields[i][0];
			var fieldType:String = fields[i][1];
			var fieldName:String = fields[i][2];
			var isOptional:Boolean= fields[i][5]
			var fieldValue:String = field.text;
			
			// Si es obligatorio
			if(isOptional == false) {
				switch(fieldType) {
				  case "text":
					if( (Validate.safeParam(fieldValue) == "") || (fieldValue.length<2) || fieldValue=="** required **") {
					  errors.push({error:default_error_message, field:field, type:fieldType, name:fieldName});
					  //setWrongField(field)
					};
					break;
				
				  case "email":
					if( !Validate.isEmail(fieldValue) ) {
					  errors.push({error:default_error_message, field:field, type:fieldType, name:fieldName});
					  //setWrongField(field)
					};
					break;
				
				  case "number":
					if( !Validate.isNumber(fieldValue) || (fieldValue == "") ) {
					  errors.push({error:default_error_message, field:field, type:fieldType, name:fieldName});
					  //setWrongField(field)
					};
					break;
				
				  default:
					if(Validate.safeParam(fieldValue) == "") {
					  errors.push({error:default_error_message, field:field, type:fieldType, name:fieldName});
					  //setWrongField(field)
					};
				};//end switch
			};//end if
		};//end-for
	}
	
	/**
	* @method 	validateCombos
	* @tooltip 	Performs validation process for ComboBoxes
	* @return	Void
	*/
	private function validateCombos():Void {
		var ccount:Number= combos.length
		for(var i:Number =0; i<ccount; i++) {
			var combo:MovieClip = combos[i][0];
			var errorVal:Object = combos[i][1];
			var errorStr:String = combos[i][2];
			var isOptional:Boolean= combos[i][3]
			var comboValue:Object = combo.getValue()
			
			// Si es obligatorio
			if(isOptional == false) {
				if( comboValue == errorVal ) {
					errorStr = (Validate.safeParam(errorStr))? errorStr : default_error_message
					errors.push({error:errorStr, field:combo, type:"combo", name:combo._name});
				};
			};//end if
		};//end-for
	}		
	
	/**
	* @method 	validateChecks
	* @tooltip 	Performs validation process for Checkboxes
	* @return	Void
	*/
	private function validateChecks():Void {
		var ccount:Number= checks.length
		for(var i:Number =0; i<ccount; i++) {
			var check:MovieClip = checks[i][0];
			var safeVal:Object = checks[i][1];
			var errorStr:String = checks[i][2];
			var isOptional:Boolean= checks[i][3]
			var checkValue:Object = check.getValue()
			
			// Si es obligatorio
			if(isOptional == false) {
				if( checkValue !== safeVal ) {
					errorStr = (Validate.safeParam(errorStr))? errorStr : default_error_message
					errors.push({error:errorStr, field:check, type:"checkbox", name:check._name});
				};
			};//end if
		};//end-for
	}		
}