import com.fwa.debug.Tracer;
import com.fwa.core.Collection;
import com.fwa.ui.Window;
import com.fwa.ui.DepthManager;

/**
* @class 	Placeholders
* @tooltip 	This class provides methods to manage Placeholders.
* 			Placeholders are MovieClips that works as holders for different levels of content.
* 			For example, you may have one Placeholder for Alerts, another for Popups, and another one for just Content.
* 			Each placeholder manages its content as Windows, and uses a DepthManager to manage those windows hierarchy.
* @author	Federico Campo Piombi
* @version	2.0
* @date 	01 Oct 2006
* 
* @see		com.wunderman.ui.Windows
*/
class com.fwa.ui.Placeholders {
	
	/**
	* @property placeholders
	* @tooltip 	Collection to keep track of the PlaceHolders. 
	*          	Each element will also be a Collection that stores elements with Windows and Depths 
	*          	for that particular PlaceHolder
	* @type 	Collection
	*/
	private static var placeholders:Collection = new Collection;
	
	/**
	* @property		remove_if_exists
	* @tooltip 		Flag: when creating a new Placeholder, this flag indicates if an existant Placeholder
	* 				should be overwritten or not
	* @type 		Boolean
	*/
	private static var remove_if_exists:Boolean = false
	
	
	/**
	* @constructor
	*/
	private function Placeholders(){}
	
	
	/**
	* @method 	createPlaceholder
	* @tooltip 	Creates a new PlaceHolder (movieclip) to use as Stack of Windows, and adds it to the PlaceHolders Collection
	* @param 	ph_name (String) Name of the PlaceHolder
	* @param 	ph_scope (MovieClip) Scope where the PlaceHolder will be created
	* @param 	ph_depth (Number) Depth of the PlaceHolder
	* @return 	MovieClip
	*/
	public static function createPlaceholder(ph_name:String, ph_scope:MovieClip, ph_depth:Number):MovieClip {
		var existant:Object = getPlaceholder(ph_name)
		if(existant == undefined || (existant!==undefined && remove_if_exists==true)){
			remove(ph_name)
			var container:MovieClip = ph_scope.createEmptyMovieClip(ph_name, ph_depth)
			var phconfig:Object = new Object({
				identifier:ph_name, 
				placeholder:container,
				places:new Collection(),
				depthMGR:new DepthManager(true)
			})
			placeholders.addElement(phconfig)
			return(container)
		}else{
			return(existant.container)
		}
	}
	
	
	/**
	* @method 	windowExists
	* @tooltip 	Returns true if the Window exists in the Placeholder's Places Collection
	* @param 	placeholder_id (String) placeholder identifier
	* @param 	window_id (String) window identifier
	* @return 	Boolean
	*/
	public static function windowExists(placeholder_id:String, window_id:String):Boolean {
		
		return ( (getWindow(placeholder_id, window_id) == undefined)? false : true )
	}
	
	
	/**
	* @method 	getWindow
	* @tooltip 	Given a placeholder identifier, and a window identifier, returns a reference to that Window
	* @param 	placeholder_id (String) placeholder identifier
	* @param 	window_id (String) window identifier
	* @return 	Window
	*/
	public static function getWindow(placeholder_id:String, window_id:String):Window {
		var places:Array = getPlaceholderElements(placeholder_id)
		debug("getWindow", "ph:"+placeholder_id+" wnd:"+window_id+" places:"+places.length)

		for(var i:Number =0; i<places.length; i++){
			if(places[i].identifier == window_id){
				return(places[i].window)
			}
		}
		return(undefined)
	}
	
	
	/**
	* @method 	getWindowPlaceholder
	* @tooltip 	Returns a reference to that Window's Placeholder
	* @param 	wnd (Window) Window to evla
	* @return 	Object
	*/
	public static function getWindowPlaceholder(wnd:Window):Object {
		var phs:Array = placeholders.getElements()

		for(var i:Number =0; i<phs.length; i++){
			var ph_id:String = phs[i].identifier
			if(windowExists(ph_id, wnd._name) == true){
				return( phs[i] )
			}
		}
		return(undefined)
	}
	
	
	/**
	* @method 	getPlaceholder
	* @tooltip 	Given a placeholder identifier, returns its reference in the Collection
	* @param 	placeholder_id (String) placeholder identifier
	* @return 	Object
	*/
	public static function getPlaceholder(placeholder_id:String):Object {
		var elements:Array = placeholders.getElements()
		for(var i:Number = 0; i<elements.length; i++){
			if(elements[i].identifier == placeholder_id){
				return(elements[i])
			}
		}
		return(undefined)
	}
	
	
	/**
	* @method 	getPlaceholderElements
	* @tooltip 	Given a placeholder identifier, returns its places
	* @param 	placeholder_id (String) placeholder identifier
	* @return 	Array
	*/
	public static function getPlaceholderElements(placeholder_id:String):Array {
		var elems:Array = getPlaceholder(placeholder_id).places.getElements()
		debug("getPlaceholderElements", "ph:"+placeholder_id+" count:"+elems.length)
		return( elems )
	}
	
	
	/**
	* @method 	getElements
	* @tooltip 	Returns an array with all elements in the placeholders Collection
	* @return	Array
	*/
	public static function getElements():Array {
		return placeholders.getElements()
	}
	
	
	/**
	* @method 	remove
	* @tooltip 	Removes a Placeholder. If Placeholder was found and removed returns TRUE, otherwise FALSE
	* @param	ph_id (String) Identifier of the Placholder to remove
	* @return	Boolean 
	*/
	public static function remove(ph_id:String):Boolean {
		var ph:Object = getPlaceholder(ph_id)
		debug("remove", "removing Placeholder: "+ph.identifier)
		if(ph!==undefined){
			//store Placeholder's reference into the Stage
			var container:MovieClip = ph.container
			
			//remove Placeholder from the Collection
			var removed_collection:Boolean = placeholders.removeElement(ph)
			debug("remove", "removing ph from collection:"+ removed_collection)
			
			//remove Placeholder from Stage
			container.swapDepths(container._parent.getNextHighestDepth())
			container.removeMovieClip()
			
			debug("remove", "remove success")
			return true
		}else{
			debug("remove", "unable to remove")
			return false
		}
	}
	
	
	/**
	* @method 	removeAll
	* @tooltip 	Removes all Placeholders in the Collection
	* @return 	Void
	*/
	public static function removeAll():Void {
		var phs:Array = getElements()
		var phs_count:Number = phs.length
		
		for(var i:Number = 0; i<phs_count; i++){
			debug("removeAll", "removing identifier:"+phs[i].identifier)
			remove(phs[i].identifier)
		}
	}
	
	/**
	* @method 	removeIfExists
	* @tooltip 	Sets the remove_if_exists flag
	* @param	val
	* @return 	Void
	*/
	public static function removeIfExists(val:Boolean):Void {
		remove_if_exists = val
	}
	
	
	//debug methods
	/**
	* @method 	list
	* @tooltip 	Traces all elements in the given placeholder
	* @param	placeholder_id (String) placeholder identifier
	* @return	Void
	*/
	public static function list(placeholder_id:String):Void {
		var ph:Object = getPlaceholder(placeholder_id)
		var placeholder:MovieClip = ph.placeholder
		for(var i in placeholder){
			trace(placeholder[i])
		}
	}
	
	/**
	* @method 	debug
	* @tooltip 	Traces a message to the output panel
	* @param	method (Strign) Method that invokes <debug>
	* @param	msg (String) message to output
	*/
	private static function debug(method:String, msg:String):Void {
		//Tracer.output(msg,method,"Placeholders")
	}
}