import mx.utils.Delegate;

/**
* This class provides the functionality to run the items sweep (Barrido).
* In order to work properly, elements must be added to the "elements" list. Once the Barrido is started
* it loops through each element of the list, waits "timeout" seconds and then loops to the next element.
* Before looping to the next element, the Barrido restores the display status of the previous element.
* 
* @version	1.7
* @since 	2 Dec 2006
*/
class com.fwaapi.Barrido {
	//properties
	private static var EVENT_OBJECT_STATECHANGE:Number = 0x800a;
	private static var _enabled:Boolean = false;
	private static var _loop:Boolean = true;
	private static var _timeout:Number = 1000;
	private static var elements:Array = new Array();
	private static var current_element:Number = -1;
	private static var previous_element:Number = -1;
	private static var int_id:Number;
	private static var mouse_lst:Object;
	private static var blocker:MovieClip;

	
	//events
	//Event executed when the Barrido finishes displaying its last element
	public static var onComplete:Function;
	
	//getter-setters
	/**
	* Gets-sets the enabled property of the Barrido
	* When disabled, it resets the last displayed element status
	* 
	* @param	val (Boolean) true if enabled.
	* @return	Boolean
	*/
	public static function get enabled():Boolean {
		return _enabled
	}
	public static function set enabled(val:Boolean):Void {
		_enabled = val
		if (val==true){
			clear();
		}
	}
	
	/**
	* Gets-sets the loop property of the Barrido, which indicates if the Barrido should start over
	* once it gets to the last element.
	* @param	val (Boolean) true if the barrido should loop.
	* @return	Boolean
	*/
	public static function get loop():Boolean {
		return _loop
	}
	public static function set loop(val:Boolean):Void {
		_loop = val
	}
	
	/**
	* Gets-Sets the timeout property of the Barrido. This property modifies the seconds between highlighted items.
	* @param 	val (Number) timeout in seconds
	* @return	Number
	*/
	public static function get timeout():Number {
		return _timeout/1000
	}
	public static function set timeout(val:Number):Void {
		_timeout = val*1000
	}
	
	
	/**
	* Constructor
	*/
	private function Barrido(){};
	
	
	/**
	* Sets the Blocker Clip and initialises its properties
	* @param	clip (MovieClip) clip to use as blocker
	* @return	Void
	*/
	public static function setBlocker(clip:MovieClip):Void {
		blocker = clip
		blocker.onRelease = Delegate.create(Barrido, Barrido.executeElementAction)
		blocker.onRollOver = Delegate.create(Mouse, Mouse.hide)
		blocker.onRollOut = blocker.onReleaseOutside = Delegate.create(Mouse, Mouse.show)
		blocker._alpha = 0
		blocker._visible = false
		blocker.useHandCursor = false
	}
	
	/**
	* Starts the Barrido.
	* Creates an interval to execute every <timeout> seconds the executeNext function.
	* The interval is not created if there are no items in the Elements array.
	* @return	Void
	*/
	public static function execute():Void {
		garbageCollector();
		var elems:Number = elements.length;
		if(elems>0 && _enabled == true){
			clear();
			executeNext();
			activateBlocker(true);
			
			if(elems>1){
				//execute the timeout only if there is more than 1 element in the list
				int_id = setInterval(Barrido.executeNext, _timeout);
			}
		}
	}
	
	/**
	* Stops the Barrido and clears the previously set interval.
	* Resets the current highlighted element to its default status.
	*/
	public static function clear():Void {
		activateBlocker(false);
		clearInterval(int_id);
		resetElement(elements[current_element]);
		current_element = -1;
		previous_element = -1;
	}
	
	/**
	* Adds an element to the Barrido list
	* @param	elem (Object) target clip
	* @param	off_frame (Object) frame number or label where the off state of the clip is
	* @param	highlight_frame (Object) frame number or label where the highlighted state of the clip is
	* @return	Void
	*/
	public static function addElement(elem:Object, off_frame:Object, highlight_frame:Object):Void {
		if(elem!==undefined){
			off_frame = (off_frame==undefined)? 1 : off_frame
			highlight_frame = (highlight_frame==undefined)? 2 : highlight_frame
			
			var obj:Object = new Object({element:elem, highlight:highlight_frame, off:off_frame})
			elements.push(obj)
		}
	}
	
	/**
	* Removes all the elements in the Barrido list
	* @return	Void
	*/
	public static function clearElements():Void {
		elements = new Array()
		current_element = previous_element = -1
	}
	
	
	/**
	* Returns a reference to the current highlighted element
	* @return	Object
	*/
	public static function getElement():Object {
		return( elements[current_element].element );
	}
	
	
	/**
	* Restores the current element's visual status, and executes the next element in the elements list.
	* @return	Void
	*/
	private static function executeNext():Void {
		//calculate previous element
		previous_element = current_element
		
		//calculate current element
		if(current_element < elements.length-1){
			current_element ++
		}else{
			if(_loop == true){
				current_element = 0
			}else{
				clear()
				onComplete()
				return //if got here, abort execution of next element
			}
		}
		
		//reset previous element
		resetElement(elements[previous_element])
		
		//rollover current element
		highlightElement(elements[current_element])
	}
	
	/**
	* Highlights the given element
	* @param	elem (Object) element to highlight
	* @return 	Void
	*/
	private static function highlightElement(elem:Object):Void {
		elem.element.gotoAndStop(elem.highlight)
		elem.element.onRollOver();
		Selection.setFocus(elem);
		//Accessibility.sendEvent(elem, 0, EVENT_OBJECT_STATECHANGE, true);
	}
	
	/**
	* Makes the given element go back to its initial state.
	* @param	elem (Object) element which state is to be reinitiated
	* @return	Void
	*/
	private static function resetElement(elem:Object):Void {
		elem.element.gotoAndStop(elem.off)
		elem.element.onRollOut();
	}
	
	/**
	* Executes the current highlighted element's onRelease function
	* @return	Void
	*/
	private static function executeElementAction(elem:Object):Void {
		var b_obj:Object = elements[current_element]
		b_obj.element.onRelease()
	}
	
	/**
	* Activates/Deactivates the Blocker clip
	* @param	status (Boolean) status of the blocker clip
	*/
	private static function activateBlocker(status:Boolean):Void {
		blocker.enabled = status
		blocker._visible = status
	}
	
	/**
	* This method loops through the elements array and deletes all elements that are undefined, which means
	* they're no longer available on Stage. This way they'll not be included in the Barrido sequence.
	* @return	Void
	*/
	private static function garbageCollector():Void {
		var elems:Number = elements.length;
		for(var i:Number = 0; i<elems; i++){
			if(elements[i].element._name == undefined){
				elements.splice(i,1)
			}
		}
	}
}