import com.fwa.system.processes.ISystemProcess;
import com.fwa.debug.Tracer;
import mx.utils.Delegate;

/**
* Static class that provides methods to manage the framework configuration System.
* The System initialisation is based on Processes. This class allows to centralize
* the individual initialisation of processes an monitor its progress.
* 
* @version	1.5
*/
class com.fwa.system.FWASystem {
	//PROPERTIES
	
	private static var inited:Boolean = false
	private static var processes:Array = new Array()
	private static var current_process:Number = -1
	
	/**
	* Constructor
	*/
	private function FWASystem(){}
	
	/**
	* Initialises the FWA Framework System
	* @return 	Void
	*/
	public static function init():Void {
		if(inited == false){
			inited = true
			System.useCodepage = true
			
			//initialize the queue
			onProcessCompleted()
			
			//execute the onSystemBegin callback
			onSystemBegin()
		}else{
			onSystemStartup(true)
		}
	}
	
	/**
	* This method shutdowns the System by restarting the init flag, and the processes list.
	* @return	Void
	*/
	public static function shutdown():Void {
		inited = false
		processes = new Array()
		current_process = -1
	}
	
	
	/**
	* Adds a process to the processes Array.
	* Returns true if the process was added successfully, or false if the System is already
	* initialised or the process is not an instance of ISystemProcess.
	* @param	process (ISystemProcess) Process to add to the list
	* @return	Boolean
	*/
	public static function addProcess(process:ISystemProcess):Boolean {
		if(inited == false){
			if(process instanceof ISystemProcess){
				processes.push(process)
				return(true)
			}else{
				Tracer.output("Invalid process!", "addProcess", "FWASystem")
				return(false)
			}
		}else{
			Tracer.output("Can't add a process when the System is initialised!", "addProcess", "FWASystem")
			return(false)
		}
	}
	
	//PRIVATE METHODS
	
	/**
	* This method receives a Process, registers its callbacks and starts it.
	* @param	process (ISystemProcess) Process to create and initialise.
	* @return 	Void
	*/
	private static function createProcess(process:ISystemProcess):Void {
		Tracer.output("Creating Process:"+process.toString(), "createProcess", "FWASystem")
		process.onProcessStarted = Delegate.create(FWASystem, FWASystem.onProcessStart);
		process.onProcessUpdated = Delegate.create(FWASystem, FWASystem.onProcessProgress);
		process.onProcessCompleted = Delegate.create(FWASystem, FWASystem.onProcessCompleted);
		process.onProcessAborted = Delegate.create(FWASystem, FWASystem.onProcessAborted);
		process.init()
	}
	
	
	/**
	* This method is executed when a process starts working
	* @param	total (Number) Total subprocesses for that Process
	* @return	Void
	*/
	private static function onProcessStart(total:Number):Void {
	}
	
	/**
	* This method is executed when a process updates one of its sub-processes.
	* It calculates the progress of the System's load, and executes the onSystemProgress callback.
	* @param	sp_current (Number) Current running subprocess
	* @param	sp_total (Number) Total subprocesses
	* @return	Void
	*/
	private static function onProcessProgress(sp_current:Number, sp_total:Number):Void {
		var p_total:Number = processes.length
		var p_current:Number = current_process+1
		
		var current_sp_percent:Number = Math.ceil(sp_current * 100 / sp_total)
		var single_sp_percent:Number = Math.ceil(100 / p_total)
		var current_sp_overall_percent:Number = single_sp_percent * current_sp_percent / 100
		var previous_sp_overall_percent:Number = (p_current-1) * single_sp_percent
		var overall_percent:Number = Math.ceil(previous_sp_overall_percent + current_sp_overall_percent)
		
		onSystemProgress(overall_percent)
	}
	
	/**
	* This method is executed when a Process sucessfully finishes its task.
	* This method evaluates if there are any remaining processes in the queue and runs them, otherwise
	* calls the onSystemStartup callback with a true value.
	* @return	Void
	*/
	private static function onProcessCompleted():Void {
		current_process+=1
		if(current_process >= processes.length){
			onSystemStartup(true)
		}else{
			createProcess(processes[current_process])
		}
	}
	
	/**
	* This method is executed when a Process is aborted by an error. It executes the onSystemStartup callback
	* with a false success param, and passes the reason as the error param.
	* @param	reason (String) Message: the reason why the Process aborted
	* @return	Void
	*/
	private static function onProcessAborted(reason:String):Void {
		Tracer.output(reason, "onProcessAborted", "FWASystem")
		onSystemStartup(false, reason)
	}
	
	
	//GETTERS
	public static function get initialised():Boolean { return inited }
	
	
	//EVENTS
	/**
	* This event is called when the System finishes initialising. The System might have been initialises
	* with or without errors at the moment this event is called, that's why there is a success flag, and an 
	* error_message string.
	* @param	success (Boolean) true if the System was initialised succesfully, otherwise false.
	* @param	error_message (String) Contains the Error Message if the System was initialised with errors.
	*/
	public static function onSystemStartup(success:Boolean, error_message:String):Void {}
	
	/**
	* This event is called when the System begins to load its processes
	*/
	public static function onSystemBegin():Void {}
	
	/**
	* This event is called on every system process/sub-process update
	* @param	percent (Number) The percent of System initialisation progress
	*/
	public static function onSystemProgress(percent:Number):Void {}

}