import com.fwaapi.Application;
import com.fwaapi.Sounds;
import com.fwaapi.Subtitles;
import com.fwa.system.FWASystem;
import com.fwa.system.FWAConfiguration;
import com.fwa.system.Configuration;
import com.fwa.system.processes.*;
import com.fwa.data.XMLConstruct;
import com.fwa.debug.Tracer;
import mx.utils.Delegate;

/**
* Static class that initialises the FWASystem hiding its main implementation.
* To initialise the System, the default language for the application and the main application path must be provided, 
* otherwise the System will fail to start.
* This class also allows the user to register the FWASystem notification callbacks.
* 
* @version	1.2
* @since 	20 Nov 2006
* 
* @ignore
* MOFIFICATION HISTORY:
* 1.1 - 10 Mar 2007
* + setLanguagePath
* + setLanguageFile
* 
* 1.2 - 17 Mar 2007
* + initLanguage
* + initApplication
* - setLanguagePath
* - setLanguageFile
*/
class com.fwaapi.SystemProxy {
	
	/**
	* Hidden Constructor
	*/
	private function SystemProxy(){}
	
	
	
	/**
	* Initialises the System language configuration and starts loading the Language file
	* @param	lFolder (String) Language folder for the application
	* @param	lFile (String) Language file for the Marco and apps list
	*/
	public static function initLanguage(lFolder:String, lFile:String):Void {
		Configuration.store("selected_language", lFolder);
		Configuration.store("selected_language_file", lFile);
		
		var oLang:Object = new XMLConstruct();
		oLang.onLoad = function(success:Boolean){
			if(success && this.applications && this.labels){
				//store into the main configuration the selected language's 
				//Applications Data and Alts Labels
				FWAConfiguration.main.applications = this.applications
				FWAConfiguration.main.labels = this.labels
				
				SystemProxy.onLanguageStartup(true);
			}else{
				SystemProxy.onLanguageStartup(false);
			}
		}
		oLang.load(lFile);
	}
	
	
	
	/**
	* Loads the given application id's language components and
	* application file
	* @param	appID (Number) ID of the application (number of node in the XML)
	*/
	public static function initApplication(appID:Number):Void {
		var oApp:Object = FWAConfiguration.main.applications.app[appID];
		Configuration.store("selected_application", oApp.folder._value);
		Configuration.store("selected_application_file", oApp.file._value);
		Configuration.store("selected_application_id", appID);
		
		Tracer.output("storing app info:" + oApp.folder._value +" - "+ oApp.file._value, "initApplication", "SystemProxy");
		
		//shutdown system
		shutdown();
		
		initSystem();
	}
	
	
	/**
	* Shutdowns the system and current application.
	* Removes all queued and playing sounds, and clears the subtitles.
	* @return	Void
	*/
	public static function shutdown():Void {
		FWASystem.shutdown();
		Application.close();
		Sounds.removeAll();
		Subtitles.clear();
	}
	
	
	/**
	* Initialises the FWASystem and the application's language.
	* @return Void
	*/
	private static function initSystem():Void {
		shutdown();
		
		//Map the FWASystem startup callback to start loading the application
		//Execute SystemProxy.onSystemStartup
		FWASystem.onSystemStartup = function(success:Boolean){
			if(success){
				Application.load();
			}
			SystemProxy.onSystemStartup(success);
		}
		
		//Map the FWASystem callbacks to the api ones
		FWASystem.onSystemProgress = Delegate.create(SystemProxy, SystemProxy.onSystemProgress);
		FWASystem.onSystemBegin = Delegate.create(SystemProxy,  SystemProxy.onSystemBegin);
		
		//Register the required initialisation Processes
		FWASystem.addProcess(new LoadHelp());
		FWASystem.addProcess(new LoadSounds());
		FWASystem.addProcess(new LoadSubtitles());
		FWASystem.addProcess(new LoadAccesibility());
		
		//initialise the System
		FWASystem.init();
	}
	
	
	//EVENTS
	/**
	* Event executed when the language file is loaded. 
	* @param	success (Boolean) true if the language file was loaded an parsed correclty
	*/
	public static function onLanguageStartup(success:Boolean):Void {}
	
	/**
	* Event executed when the System has finished all its processes
	* @param 	success (Boolean) true if the System has initialised with no errors
	*/
	public static function onSystemStartup(success:Boolean, error_message:String):Void {}
	
	/**
	* @see	com.fwa.system.FWASystem  [onSystemBegin]
	*/
	public static function onSystemBegin():Void {}
	
	/**
	* @see	com.fwa.system.FWASystem  [onSystemProgress]
	*/
	public static function onSystemProgress(percent:Number):Void {}
}